@@ -16,8 +16,9 @@ import json
 @apply_opps_rules('channels')
 class ChannelAdmin(PublishableAdmin, MPTTModelAdmin):
     prepopulated_fields = {"slug": ("name",)}
-    list_display = ['name', 'parent', 'site', 'date_available', 'homepage',
-                    'order', 'show_in_menu', 'published']
+    list_display = ['name', 'show_channel_path', 'parent', 'site',
+                    'date_available', 'homepage', 'order', 'show_in_menu',
+                    'published']
     list_filter = ['date_available', 'published', 'site', 'homepage', 'parent',
                    'show_in_menu']
     search_fields = ['name', 'slug', 'long_slug', 'description']
@@ -27,15 +28,19 @@ class ChannelAdmin(PublishableAdmin, MPTTModelAdmin):
 
     fieldsets = (
         (_(u'Identification'), {
-            'fields': ('site', 'parent', 'name', 'slug', 'layout',
+            'fields': ('site', 'parent', 'name', 'slug', 'layout', 'hat',
                        'description', 'order', ('show_in_menu',
                                                 'include_in_main_rss'),
-                       'homepage', 'group')}),
+                       'homepage', 'group', 'paginate_by')}),
         (_(u'Publication'), {
             'classes': ('extrapretty'),
             'fields': ('published', 'date_available')}),
     )
 
+    def show_channel_path(self, obj):
+        return unicode(obj)
+    show_channel_path.short_description = _(u'Channel Path')
+
     def save_model(self, request, obj, form, change):
         long_slug = u"{}".format(obj.slug)
         if obj.parent:
