@@ -10,6 +10,7 @@ from django.core.cache import cache
 from django.utils import timezone
 from django.conf import settings
 from django.template.defaultfilters import slugify
+from django.contrib.sites.models import Site
 
 from polymorphic import PolymorphicModel
 from polymorphic.showfields import ShowFieldContent
@@ -21,7 +22,7 @@ from opps.db.models.fields import JSONField
 from opps.boxes.models import BaseBox
 
 from .signals import shorturl_generate, delete_container
-from .tasks import check_mirror_channel
+from .tasks import check_mirror_channel, check_mirror_site
 
 
 class Container(PolymorphicModel, ShowFieldContent, Publishable, Slugged,
@@ -96,11 +97,17 @@ class Container(PolymorphicModel, ShowFieldContent, Publishable, Slugged,
         models.signals.post_save.connect(shorturl_generate,
                                          sender=self.__class__)
         super(Container, self).save(*args, **kwargs)
-        if settings.OPPS_MIRROR_CHANNEL and self.mirror_channel:
+        if settings.OPPS_MIRROR_CHANNEL and (
+            self.mirror_channel or self.mirror_site)\
+                and self.child_class != u"Mirror":
             check_mirror_channel.delay(
                 container=self, Mirror=Mirror)
+            check_mirror_site.delay(
+                container=self, Mirror=Mirror)
 
     def get_absolute_url(self):
+        if self.channel.homepage:
+            return u"/{}.html".format(self.slug)
         return u"/{}/{}.html".format(self.channel_long_slug, self.slug)
 
     def get_thumb(self):
@@ -217,6 +224,19 @@ class ContainerBox(BaseBox):
         blank=True,
         max_length=250,
     )
+    main_image = models.ForeignKey(
+        'images.Image',
+        null=True, blank=True,
+        on_delete=models.SET_NULL,
+        verbose_name=_(u'Main Image'),
+    )
+    main_image_caption = models.CharField(
+        _(u"Main Image Caption"),
+        max_length=4000,
+        blank=True,
+        null=True,
+        help_text=_(u'Maximum characters 4000'),
+    )
     containers = models.ManyToManyField(
         'containers.Container',
         null=True, blank=True,
@@ -231,33 +251,120 @@ class ContainerBox(BaseBox):
         verbose_name=_(u'Query Set')
     )
 
+    content_group = models.CharField(
+        _(u"Content Group"),
+        max_length=255,
+        default='default',
+        help_text=_(u"Boxes in the same group do not allow repetitions")
+    )
+
     class Meta:
         verbose_name = _(u'Container box')
         verbose_name_plural = _(u'Containers boxes')
 
-    def ordered_containers(self, field='order'):
-        now = timezone.now()
-        return self.containers.filter(
-            models.Q(containerboxcontainers__date_end__gte=now) |
-            models.Q(containerboxcontainers__date_end__isnull=True),
-            published=True,
-            date_available__lte=now,
-            containerboxcontainers__date_available__lte=now
-        ).order_by('containerboxcontainers__order').distinct()
+    def __init__(self, *args, **kwargs):
+        """
+        to avoid re-execution of methods
+        its results are cached in a local storage
+        per instance cache
+        """
+        super(ContainerBox, self).__init__(*args, **kwargs)
+        if not hasattr(self, 'local_cache'):
+            self.local_cache = {}
+
+    @property
+    def has_content(self):
+        # TODO: should check start/end available_dates
+        if self.containerboxcontainers_set.exists():
+            return True
+        qs = self.get_queryset(use_local_cache=False)
+        if qs and qs.exists():
+            return True
+
+    def ordered_containers(self, field='order', exclude_ids=None):
+        cached = self.local_cache.get('ordered_containers', None)
+        if cached:
+            return cached
+
+        exclude_ids = exclude_ids or []
 
-    def ordered_box_containers(self):
         now = timezone.now()
-        return self.containerboxcontainers_set.filter(
+        fallback = getattr(settings, 'OPPS_MULTISITE_FALLBACK', False)
+        if not fallback:
+            qs = self.containers.filter(
+                models.Q(containerboxcontainers__date_end__gte=now) |
+                models.Q(containerboxcontainers__date_end__isnull=True),
+                published=True,
+                date_available__lte=now,
+                containerboxcontainers__date_available__lte=now
+            ).exclude(
+                id__in=exclude_ids
+            ).order_by('containerboxcontainers__order').distinct()
+            #[exclude_ids.append(i.id) for i in qs if not i.id in exclude_ids]
+        else:
+            site_master = Site.objects.order_by('id')[0]
+            boxes = [self]
+            if fallback and site_master != self.site:
+                try:
+                    master_box = self.__class__.objects.get(
+                        site=site_master, slug=self.slug
+                    )
+                    boxes.insert(0, master_box)
+                except self.__class__.DoesNotExist:
+                    pass
+
+            qs = ContainerBoxContainers.objects.filter(
+                models.Q(date_end__gte=now) |
+                models.Q(date_end__isnull=True),
+                models.Q(container__published=True,
+                         container__date_available__lte=now) |
+                models.Q(container__isnull=True),
+                containerbox__in=boxes,
+                date_available__lte=now,
+            ).exclude(
+                container__id__in=exclude_ids
+            ).order_by('-containerbox__site', 'order').distinct()
+
+        self.local_cache['ordered_containers'] = qs
+        return qs
+
+    def ordered_box_containers(self, exclude_ids=None):
+
+        fallback = getattr(settings, 'OPPS_MULTISITE_FALLBACK', False)
+        if fallback:
+            return self.ordered_containers(exclude_ids=exclude_ids)
+
+        cached = self.local_cache.get('ordered_box_containers')
+        if cached:
+            return cached
+
+        exclude_ids = exclude_ids or []
+
+        now = timezone.now()
+        qs = self.containerboxcontainers_set.filter(
             models.Q(date_end__gte=now) |
             models.Q(date_end__isnull=True),
             date_available__lte=now
+        ).exclude(
+            container__id__in=exclude_ids
         ).order_by('order').distinct()
 
-    def get_queryset(self):
-        """
-        for backwards compatibility
-        """
-        return self.queryset and self.queryset.get_queryset()
+        self.local_cache['ordered_box_containers'] = qs
+        return qs
+
+    def get_queryset(self, exclude_ids=None, use_local_cache=True):
+        cached = self.local_cache.get('get_queryset')
+        if use_local_cache and cached:
+            return cached
+
+        queryset = self.queryset and self.queryset.get_queryset(
+            content_group=self.content_group,
+            exclude_ids=exclude_ids,
+            use_local_cache=use_local_cache
+        )
+        if use_local_cache:
+            self.local_cache['get_queryset'] = queryset
+        return queryset
 
 
 class ContainerBoxContainers(models.Model):
@@ -304,7 +411,7 @@ class ContainerBoxContainers(models.Model):
         help_text=_(u'Maximum characters 4000'),
     )
 
-    url = models.URLField(_(u"URL"), null=True, blank=True)
+    url = models.CharField(_(u"URL"), max_length=255, null=True, blank=True)
 
     class Meta:
         ordering = ('order',)
@@ -333,5 +440,8 @@ class Mirror(Container):
                                   related_name='containers_mirror',
                                   verbose_name=_(u'Container'))
 
+    class Meta:
+        verbose_name = _(u'Mirror')
+        verbose_name_plural = _(u'Mirrors')
 
 models.signals.post_delete.connect(delete_container, sender=Container)
