@@ -7,10 +7,14 @@ from django.utils import timezone
 from django.utils.safestring import mark_safe
 from django.template.defaultfilters import linebreaksbr
 from django.core.cache import cache
+from django.contrib.sites.models import Site
 
-from opps.containers.models import Container, ContainerBox
+from opps.contrib.middleware.global_request import get_request
+from opps.containers.models import Container, ContainerBox, Mirror
 from opps.channels.models import Channel
 
+from magicdate import magicdate
+
 
 register = template.Library()
 logger = logging.getLogger()
@@ -47,37 +51,132 @@ def get_recommendations(query_slice, child_class, container):
     return container.recommendation(child_class, bits)
 
 
-@register.simple_tag(takes_context=True)
-def get_containerbox(context, slug, template_name=None):
+@register.assignment_tag(takes_context=True)
+def load_boxes(context, slugs=None, **filters):
+    if slugs:
+        filters['slug__in'] = ordered_slugs = slugs.split(',')
 
     request = context['request']
+    current_site = getattr(
+        request,
+        'site',
+        Site.objects.get(pk=settings.SITE_ID)
+    )
 
-    cachekey = "ContainerBox-{}-{}-{}".format(
+    filters['site__in'] = [current_site]
+    master_site = settings.OPPS_CONTAINERS_SITE_ID or 1
+    if current_site.id != master_site:
+        filters['site__in'].append(master_site)
+
+    filters['date_available__lte'] = timezone.now()
+    filters['published'] = True
+
+    boxes = ContainerBox.objects.filter(**filters).order_by('-site')
+    fallback = getattr(settings, 'OPPS_MULTISITE_FALLBACK', False)
+
+    exclude_ids = []
+
+    if slugs:
+        ob = lambda i, o=ordered_slugs: (
+            i.site != current_site, i.site.id, o.index(i.slug)
+        )
+
+        boxes = sorted(
+            boxes,
+            key=ob
+        )
+
+    for box in boxes:
+        if box.queryset:
+            results = box.get_queryset(exclude_ids=exclude_ids)
+        else:
+            results = box.ordered_containers(exclude_ids=exclude_ids)
+
+        if box.queryset:
+            [exclude_ids.append(i.pk)
+             for i in results
+             if not i.pk in exclude_ids
+             and issubclass(i.__class__, Container)]
+        elif fallback:
+            [exclude_ids.append(i.container_id)
+             for i in results
+             if i.container_id and not i.container_id in exclude_ids]
+        else:
+            [exclude_ids.append(i.pk)
+             for i in results
+             if not i.pk in exclude_ids]
+
+    results = {}
+    for box in boxes:
+        if not box.slug in results:
+            results[box.slug] = box
+
+    get_request().container_boxes = results
+    return results
+
+
+@register.simple_tag(takes_context=True)
+def get_containerbox(context, slug, template_name=None, **extra_context):
+
+    request = context['request']
+    current_site = getattr(
+        request,
+        'site',
+        Site.objects.get(pk=settings.SITE_ID)
+    )
+    is_mobile = getattr(request, 'is_mobile', False)
+
+    cachekey = "ContainerBox-{}-{}-{}-{}".format(
         slug,
         template_name,
-        request.is_mobile,
+        is_mobile,
+        current_site.id
     )
 
     render = cache.get(cachekey)
     if render:
         return render
 
-    try:
-        box = ContainerBox.objects.get(site=settings.SITE_ID, slug=slug,
-                                       date_available__lte=timezone.now(),
-                                       published=True)
-    except ContainerBox.DoesNotExist:
-        box = None
+    box = getattr(get_request(), 'container_boxes', {}).get(slug, None)
+
+    if not box:
+        filters = {}
+        filters['site'] = current_site
+        filters['slug'] = slug
+        filters['date_available__lte'] = timezone.now()
+        filters['published'] = True
+
+        master_site = settings.OPPS_CONTAINERS_SITE_ID or 1
+
+        try:
+            box = ContainerBox.objects.get(**filters)
+        except ContainerBox.DoesNotExist:
+            box = None
+
+        if current_site.id != master_site and \
+           not box or not getattr(box, 'has_content', False):
+            filters['site'] = master_site
+            try:
+                box = ContainerBox.objects.get(**filters)
+            except ContainerBox.DoesNotExist:
+                box = None
+
+        if not box:
+            box = ContainerBox.objects.none()
 
     t = template.loader.get_template('articles/articlebox_detail.html')
     if template_name:
         t = template.loader.get_template(template_name)
 
-    render = t.render(template.Context({
+    context = {
         'articlebox': box,
         'slug': slug,
-        'context': context}
-    ))
+        'context': context
+    }
+
+    context.update(extra_context)
+
+    render = t.render(template.Context(context))
 
     cache.set(cachekey, render, settings.OPPS_CACHE_EXPIRE)
 
@@ -107,14 +206,17 @@ def get_all_containerbox(channel_long_slug=None, template_name=None):
     if render:
         return render
 
-    boxes = ContainerBox.objects.filter(
-        site=settings.SITE_ID,
-        date_available__lte=timezone.now(),
-        published=True)
+    filters = {}
+    filters['date_available__lte'] = timezone.now()
+    filters['published'] = True
+    filters['site'] = settings.SITE_ID
+    if settings.OPPS_CONTAINERS_SITE_ID:
+        filters['site'] = settings.OPPS_CONTAINERS_SITE_ID
+
+    boxes = ContainerBox.objects.filter(**filters)
 
     if channel_long_slug:
-        boxes = boxes.filter(
-            channel_long_slug=channel_long_slug)
+        boxes = boxes.filter(channel_long_slug=channel_long_slug)
 
     t = template.loader.get_template('articles/articlebox_list.html')
     if template_name:
@@ -127,7 +229,7 @@ def get_all_containerbox(channel_long_slug=None, template_name=None):
 
 
 @register.simple_tag
-def get_post_content(post, template_name='articles/post_related.html',
+def get_post_content(post, template_name='containers/post_related.html',
                      content_field='content', related_name='related_posts',
                      get_related=True, safe=True, divider="<br />",
                      placeholder=settings.OPPS_RELATED_POSTS_PLACEHOLDER):
@@ -160,6 +262,11 @@ def get_post_content(post, template_name='articles/post_related.html',
     # REMOVE NEW LINES
     content = linebreaksbr(content)
 
+    # Fix embed allowfullscreen
+    # TinyMCE BUG
+    content = content.replace('allowfullscreen="allowfullscreen"',
+                              'allowfullscreen="true"')
+
     if not get_related:
         return content
 
@@ -211,13 +318,94 @@ def get_url(obj, http=False, target=None, url_only=False):
 @register.assignment_tag
 def get_containers_by(limit=None, **filters):
     """Return a list of containers filtered by given args"""
-    cachekey = hash(frozenset(filters.items()))
+    cachekey = u'getcontainersby-{}'.format(hash(frozenset(filters.items())))
     _cache = cache.get(cachekey)
     if _cache:
         return _cache
+
+    site = settings.SITE_ID
+    if settings.OPPS_CONTAINERS_SITE_ID:
+        site = settings.OPPS_CONTAINERS_SITE_ID
+
     containers = [i for i in Container.objects.filter(
-        site=settings.SITE_ID, published=True, **filters)[:limit]]
-    cache.set("getconby-{}".format(cachekey), 3600)
+        site=site, published=True, **filters)[:limit]]
+
+    cache.set(cachekey, containers, 3600)
+    return containers
+
+
+@register.assignment_tag
+def filter_queryset_by(queryset, **filters):
+    """Filter object list"""
+    if not getattr(queryset, 'query', False):
+        return queryset
+
+    cachekey = u'filterquerysetby-{}'.format(hash(unicode(queryset.query)))
+    _cache = cache.get(cachekey)
+    if _cache:
+        return _cache
+
+    # check for __in lookups and split the param
+    found_in_lookup = None
+    for key in filters.keys():
+        if '__in' in key:
+            found_in_lookup = key
+
+    if found_in_lookup:
+        filters[found_in_lookup] = filters[found_in_lookup].split(',')
+
+    if not queryset.query.can_filter():
+        # create new queryset based on the ids and apply filter
+        ids = [i.id for i in queryset]
+        queryset = queryset.model.objects.filter(id__in=ids).filter(**filters)
+        return queryset
+
+    containers = queryset.filter(**filters)
+    cache.set(cachekey, containers, 3600)
+    return containers
+
+
+@register.assignment_tag
+def exclude_queryset_by(queryset, **excludes):
+    """Exclude object list"""
+
+    if not getattr(queryset, 'query', False):
+        return queryset
+
+    cachekey = u'excludequerysetby-{}'.format(hash(unicode(queryset.query)))
+    _cache = cache.get(cachekey)
+    if _cache:
+        return _cache
+
+    # check for __in lookups and split the param
+    found_in_lookup = None
+    for key in excludes.keys():
+        if '__in' in key:
+            found_in_lookup = key
+
+    if found_in_lookup:
+        excludes[found_in_lookup] = excludes[found_in_lookup].split(',')
+
+    if not queryset.query.can_filter():
+        # create new queryset based on the ids and apply filter
+        ids = [i.id for i in queryset]
+        containers = queryset.model.objects.filter(id__in=ids).exclude(
+            **excludes
+        )
+    else:
+        containers = queryset.exclude(**excludes)
+
+    if 'child_class' in excludes:
+        # we need to exclude the mirrors containing the child_class that
+        # we want to exclude
+        bad_child_class = excludes['child_class']
+        mirrors = Mirror.objects.filter(
+            container__child_class=bad_child_class
+        ).values_list('id', flat=True)
+        if mirrors:
+            containers = containers.exclude(pk__in=mirrors)
+
+    cache.set(cachekey, containers, 3600)
     return containers
 
 
@@ -225,6 +413,14 @@ def get_containers_by(limit=None, **filters):
 def get_container_by_channel(slug, number=10, depth=1,
                              include_children=True, **kwargs):
     box = None
+    magic_date = kwargs.pop('magic_date', False)
+    date = timezone.now()
+    if magic_date:
+        try:
+            date = magicdate(magic_date)
+        except Exception:
+            pass
+
     if include_children:
         try:
             kwargs['channel_long_slug__in'] = cache.get(
@@ -256,8 +452,10 @@ def get_container_by_channel(slug, number=10, depth=1,
             kwargs['channel_long_slug__in'] = []
     try:
         kwargs['site'] = settings.SITE_ID
+        if settings.OPPS_CONTAINERS_SITE_ID:
+            kwargs['site'] = settings.OPPS_CONTAINERS_SITE_ID
         kwargs['show_on_root_channel'] = include_children
-        kwargs['date_available__lte'] = timezone.now()
+        kwargs['date_available__lte'] = date
         kwargs['published'] = True
         box = Container.objects.distinct().filter(
             **kwargs).order_by('-date_available')[:number]
@@ -269,7 +467,10 @@ def get_container_by_channel(slug, number=10, depth=1,
 @register.assignment_tag
 def get_containerbox_by(**filters):
     """Return a list of containers filtered by given args"""
-    return ContainerBox.objects.filter(site=settings.SITE_ID,
+    site = settings.SITE_ID
+    if settings.OPPS_CONTAINERS_SITE_ID:
+        site = settings.OPPS_CONTAINERS_SITE_ID
+    return ContainerBox.objects.filter(site=site,
                                        published=True,
                                        date_available__lte=timezone.now(),
                                        **filters)
@@ -292,9 +493,12 @@ def get_containerbox_list(context, slug, num=0, template_name=None):
     if render:
         return render
 
+    site = settings.SITE_ID
+    if settings.OPPS_CONTAINERS_SITE_ID:
+        site = settings.OPPS_CONTAINERS_SITE_ID
     try:
         box = ContainerBox.objects.filter(
-            site=settings.SITE_ID, slug=slug,
+            site=site, slug=slug,
             date_available__lte=timezone.now(),
             published=True)
         if isinstance(num, int) and num > 0 and box:
@@ -316,3 +520,17 @@ def get_containerbox_list(context, slug, num=0, template_name=None):
     cache.set(cachekey, render, settings.OPPS_CACHE_EXPIRE)
 
     return render
+
+
+@register.assignment_tag
+def get_custom_field_value(obj, field_slug):
+    """
+    Return a custom field value
+    """
+    if not callable(getattr(obj, 'custom_fields')):
+        return None
+
+    if not obj.custom_fields():
+        return None
+
+    return obj.custom_fields().get(field_slug)
