@@ -18,8 +18,9 @@ def image_obj(image, **kwargs):
     HALIGN_VALUES = ("left", "center", "right")
     VALIGN_VALUES = ("top", "middle", "bottom")
 
-    if image == "":
+    if image == "" or not image:
         return ""
+
     if settings.THUMBOR_ENABLED:
         new = {}
         new['flip'] = image.flip
@@ -40,4 +41,7 @@ def image_obj(image, **kwargs):
 
         kwargs = dict(new, **kwargs)
 
-    return url(image_url=image.archive.url, **kwargs)
+    if image.archive_link and image.archive_link != "":
+        return url(image_url=image.archive_link, **kwargs)
+
+    return image.image_url(**kwargs)
