@@ -3,4 +3,4 @@
 import haystack
 
 
-haystack.autodiscover()
+#haystack.autodiscover()
