@@ -1,5 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+from django.http import Http404
 from django.shortcuts import get_object_or_404
 from django.utils import timezone
 from django import template
@@ -35,9 +36,11 @@ class View(object):
                                                        queryset.model.
                                                        __name__).upper()
 
-        paginate_by = getattr(settings, setting_name, self.paginate_by)
+        by_settings = getattr(settings, setting_name, self.paginate_by)
+        by_request = self.request.GET.get('paginate_by')
+        by_channel = getattr(self.channel, 'paginate_by', None)
 
-        return paginate_by
+        return by_request or by_channel or by_settings
 
     def get_context_data(self, **kwargs):
         if not self.long_slug:
@@ -86,6 +89,8 @@ class View(object):
         if self.channel:
             context['channel'] = self.channel
 
+        context['breadcrumb'] = self.get_breadcrumb()
+
         if self.slug:
             try:
                 context['next'] = self.get_object()\
@@ -104,11 +109,15 @@ class View(object):
             if self.get_object().child_class == 'Mirror':
                 context['context'] = self.get_object().container
 
+        if self.request.META.get('HTTP_X_PJAX', False) or\
+           self.request.is_ajax():
+            context['extends_parent'] = 'base_ajax.html'
+
         return context
 
     def get_template_folder(self):
         domain_folder = "containers"
-        if settings.SITE_ID > 1:
+        if self.site.id > 1:
             domain_folder = "{}/containers".format(self.site.domain)
         return domain_folder
 
@@ -123,17 +132,39 @@ class View(object):
         return self.long_slug
 
     def set_channel_rules(self):
-        self.channel = get_object_or_404(Channel,
-                                         site__domain=self.site.domain,
-                                         long_slug=self.long_slug,
-                                         date_available__lte=timezone.now(),
-                                         published=True)
+        self.fallback = getattr(settings, 'OPPS_MULTISITE_FALLBACK', False)
+        filters = dict(
+            site__domain=self.site.domain,
+            long_slug=self.long_slug,
+            date_available__lte=timezone.now(),
+            published=True
+        )
+
+        try:
+            self.channel = Channel.objects.get(**filters)
+        except Channel.DoesNotExist:
+            if not self.fallback or self.site == self.site_master:
+                raise Http404('Channel not found and fallback disabled')
+            filters['site__domain'] = self.site_master.domain
+            self.channel = get_object_or_404(Channel, **filters)
+
         self.long_slug = self.channel.long_slug
 
         self.channel_long_slug = [self.long_slug]
-        for children in self.channel.get_descendants(include_self=False):
+        self.channel_descendants = self.channel.get_descendants(
+            include_self=False)
+        for children in self.channel_descendants:
             self.channel_long_slug.append(children.long_slug)
 
+    def get_breadcrumb(self):
+        try:
+            if self.channel.is_root_node():
+                return []
+        except:
+            return []
+
+        return self.channel.get_ancestors(include_self=True)
+
     def check_template(self, _template):
         try:
             template.loader.get_template(_template)
